# -*- coding: utf-8 -*-
# pylint: disable=W0614,W0611,W0622
# flake8: noqa
# isort:skip_file
import logging
import sys

logger = logging.getLogger('adsorption_file_parser')
logger.setLevel(logging.DEBUG)

# create console handler
ch = logging.StreamHandler(stream=sys.stdout)
ch.setLevel(logging.INFO)

# add the handlers to the logger
logger.addHandler(ch)

# Solve version shenanigans on conda-build
try:
    # -- Distribution mode --
    # import from _version.py generated by setuptools_scm during release
    from ._version import __version__
except ImportError:
    # -- Source mode --
    # use setuptools_scm to get the current version from src using git
    from setuptools_scm import get_version as _gv
    from os import path as _path
    __version__ = _gv(_path.join(_path.dirname(__file__), _path.pardir))


class ParsingError(Exception):
    """Raised when parsing fails."""


_SUPPORTED_FORMATS = {
    'smsdvs': ('xlsx', ),
    'bel': ('csv', 'xl', 'dat'),
    'mic': ('xl', ),
    '3p': ('xl'),  # 'jwgbt'),
    'qnt': ('txt-raw', ),
    'generic': ('csv'),
}


def read(path, manufacturer, fmt, **options):
    """
    Parse a file generated by commercial apparatus.

    Parameters
    ----------
    path: str
        the location of the file.
    manufacturer : {'mic', 'bel', '3p'}
        Manufacturer of the apparatus.
    fmt : {'xl', 'txt', ...}
        The format of the import for the isotherm.

    Returns
    -------
    meta: dict
        the metadata
    data: dict
        all available data
    """

    if manufacturer not in _SUPPORTED_FORMATS:
        raise ParsingError(
            f'Currently available manufacturers are {list(_SUPPORTED_FORMATS.keys())})'
        )

    if fmt not in _SUPPORTED_FORMATS[manufacturer]:
        raise ParsingError(f'Currently available formats are {_SUPPORTED_FORMATS[manufacturer]}')

    if manufacturer == 'smsdvs' and fmt == 'xlsx':
        from .sms_dvs_excel import parse
    elif manufacturer == 'mic' and fmt == 'xl':
        from .mic_excel import parse
    elif manufacturer == 'bel':
        if fmt == 'xl':
            from .bel_excel import parse
        elif fmt == 'csv':
            from .bel_csv import parse
        elif fmt == 'dat':
            from .bel_dat import parse
    elif manufacturer == '3p':
        if fmt == 'xl':
            from .trp_excel import parse
        elif fmt == 'jwgbt':
            from .trp_xml import parse
    elif manufacturer == 'qnt' and fmt == 'txt-raw':
        from .qnt_txt import parse
    elif manufacturer == 'generic':
        if fmt == 'csv':
            from .generic_csv import parse
    else:
        raise ParsingError('Something went wrong.')

    return parse(path, **options)
